@@ -11,8 +11,6 @@ to standard output.
 
 """
 
-from __future__ import with_statement
-
 import ctypes
 import struct
 import sys
@@ -22,15 +20,17 @@ def _uptime_linux():
     """Returns uptime in seconds or None, on Linux."""
     # With procfs
     try:
-        with open('/proc/uptime', 'r') as f:
-            return float(f.readline().split()[0])
+        f = open('/proc/uptime', 'r')
+        up = float(f.readline().split()[0])
+        f.close()
+        return up
     except (IOError, ValueError):
         pass
 
     # Without procfs (really?)
     try:
         libc = ctypes.CDLL('libc.so')
-    except OSError:
+    except (OSError, RuntimeError):
         # Debian and derivatives do the wrong thing because /usr/lib/libc.so
         # is a GNU ld script rather than an ELF object. To get around this, we
         # have to be more specific.
@@ -39,7 +39,7 @@ def _uptime_linux():
         # this point we're already pretty sure this isn't Linux.
         try:
             libc = ctypes.CDLL('libc.so.6')
-        except OSError:
+        except (OSError, RuntimeError):
             return None
 
     if not hasattr(libc, 'sysinfo'):
@@ -53,16 +53,26 @@ def _uptime_linux():
     up = struct.unpack('@l', buf.raw[:struct.calcsize('@l')])[0]
     return up if up >= 0 else None
 
+def _uptime_beos():
+    """Returns uptime in seconds on None, on BeOS/Haiku."""
+    try:
+        libroot = ctypes.CDLL('libroot.so')
+    except (OSError, RuntimeError):
+        return None
+
+    libroot.system_time.restype = ctypes.c_int64
+    return libroot.system_time() / 1000000.
+
 def _uptime_bsd():
     """Returns uptime in seconds or None, on BSD (including OS X)."""
     try:
         libc = ctypes.CDLL('libc.so')
-    except OSError:
+    except (OSError, RuntimeError):
         # OS X; can't use ctypes.util.find_library because that creates
         # a new process on Linux, which is undesirable.
         try:
             libc = ctypes.CDLL('libc.dylib')
-        except OSError:
+        except (OSError, RuntimeError):
             return None
     
     if not hasattr(libc, 'sysctlbyname'):
@@ -97,14 +107,15 @@ def _uptime_plan9():
     # support. Maybe some Linuxes implement /dev/time, though, someone was
     # talking about it somewhere.
     try:
-        with open('/dev/time', 'r') as f:
-            # The time file holds one 32-bit number representing the sec-
-            # onds since start of epoch and three 64-bit numbers, repre-
-            # senting nanoseconds since start of epoch, clock ticks, and
-            # clock frequency.
-            #  -- cons(3)
-            s, ns, ct, cf = f.read().split()
-            return float(ct) / float(cf)
+        # The time file holds one 32-bit number representing the sec-
+        # onds since start of epoch and three 64-bit numbers, repre-
+        # senting nanoseconds since start of epoch, clock ticks, and
+        # clock frequency.
+        #  -- cons(3)
+        f = open('/dev/time', 'r')
+        s, ns, ct, cf = f.read().split()
+        f.close()
+        return float(ct) / float(cf)
     except (IOError, ValueError):
         return None
 
@@ -112,7 +123,7 @@ def _uptime_solaris():
     """Returns uptime in seconds or None, on Solaris."""
     try:
         kstat = ctypes.CDLL('libkstat.so')
-    except OSError:
+    except (OSError, RuntimeError):
         return None
 
     # kstat doesn't have uptime, but it does have boot time.
@@ -195,28 +206,15 @@ def _uptime_windows():
 
 def uptime():
     """Returns uptime in seconds if even remotely possible, or None if not."""
-    if sys.platform == 'linux2':
-        up = _uptime_linux()
-        if up is not None:
-            return up
-
-    if sys.platform == 'win32':
-        up = _uptime_windows()
-        if up is not None:
-            return up
-
-    if sys.platform == 'darwin' or 'bsd' in sys.platform:
-        up = _uptime_bsd()
-        if up is not None:
-            return up
-
-    if sys.platform == 'sunos5':
-        up = _uptime_solaris()
-        if up is not None:
-            return up
-
-    return _uptime_bsd() or _uptime_plan9() or \
-           _uptime_linux() or _uptime_windows() or _uptime_solaris()
+    return {'beos5': _uptime_beos,
+            'cygwin': _uptime_linux,
+            'darwin': _uptime_osx,
+            'haiku1': _uptime_beos,
+            'linux2': _uptime_linux,
+            'sunos5': _uptime_solaris,
+            'win32': _uptime_windows}.get(sys.platform, _uptime_bsd)() or \
+           _uptime_bsd() or _uptime_plan9() or _uptime_linux() or \
+           _uptime_windows() or _uptime_solaris() or _uptime_beos()
 
 
 if __name__ == '__main__':
